package com.atlassian.elasticsearch.buckler;

import com.atlassian.elasticsearch.buckler.config.BucklerConfig;
import com.atlassian.elasticsearch.buckler.security.AuthRateLimiter;
import org.elasticsearch.common.util.concurrent.ThreadContext;
import org.elasticsearch.transport.TransportInterceptor;
import org.elasticsearch.transport.TransportRequest;
import org.elasticsearch.transport.TransportRequestHandler;

/**
 * Basic authentication for TCP transport.
 */
public class AuthTransportInterceptor implements TransportInterceptor {

    private final AuthRateLimiter authRateLimiter;
    private final BucklerConfig config;
    private final ThreadContext threadContext;

    public AuthTransportInterceptor(AuthRateLimiter authRateLimiter,
                                    BucklerConfig config,
                                    ThreadContext threadContext) {
        this.authRateLimiter = authRateLimiter;
        this.config = config;
        this.threadContext = threadContext;
    }

    @Override
    public <T extends TransportRequest> TransportRequestHandler<T> interceptHandler(String action, String executor,
                                                boolean forceExecution, TransportRequestHandler<T> actualHandler) {
        if (config.getAuthConfig().isEnabledForTcp()) {
            return new AuthTransportRequestHandler<>(actualHandler, config, authRateLimiter, threadContext);
        }
        return actualHandler;
    }

    @Override
    public AsyncSender interceptSender(AsyncSender sender) {
        return new AuthorisedSender(sender, config, threadContext);
    }
}
