package com.atlassian.elasticsearch.buckler.config;

import org.apache.logging.log4j.Logger;
import org.elasticsearch.client.node.NodeClient;
import org.elasticsearch.common.logging.ESLoggerFactory;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.xcontent.XContent;
import org.elasticsearch.common.xcontent.XContentFactory;
import org.elasticsearch.common.xcontent.XContentParser;
import org.elasticsearch.rest.*;

import java.io.IOException;
import java.util.Map;

public class ConfigRestHandler extends BaseRestHandler {

    private static final Logger logger = ESLoggerFactory.getLogger(ConfigRestHandler.class.getName());

    private static final String AUTH_BASIC_PASSWORD_PATH = "/_buckler/config/auth.basic.password";

    private final BucklerConfig config;

    public ConfigRestHandler(Settings settings, RestController controller, BucklerConfig config) {
        super(settings);
        this.config = config;

        controller.registerHandler(RestRequest.Method.PUT, AUTH_BASIC_PASSWORD_PATH, this);

    }

    @Override
    protected RestChannelConsumer prepareRequest(RestRequest request, NodeClient client) throws IOException {
        return handleAuthBasicPassword(request);
    }

    private RestChannelConsumer handleAuthBasicPassword(RestRequest request) throws IOException {
        Map<String, Object> data;
        XContent xContent = XContentFactory.xContent(request.content());
        try (XContentParser parser = xContent.createParser(request.getXContentRegistry(), request.content())) {
            data = parser.map();
        }
        Object valueObject = data.get("value");
        if (!(valueObject instanceof String)) {
            return channel -> channel.sendResponse(new BytesRestResponse(RestStatus.BAD_REQUEST, "value must be a string"));
        }

        String value = (String) valueObject;
        try {
            config.updateAuthPassword(value);
            return channel -> channel.sendResponse(new BytesRestResponse(RestStatus.OK, "OK"));
        } catch (AuthConfigUpdateException e) {
            logger.error("Updating basic authentication password failed", e);
            return channel -> channel.sendResponse(new BytesRestResponse(channel, RestStatus.INTERNAL_SERVER_ERROR, e));
        }
    }
}
